// SqlAutmn.h


#ifndef __SQLAUTMN_H__
#define __SQLAUTMN_H__

#pragma pack(push, 4)

#ifdef SQLAUTMN_EXPORTS
# define SQLCLASS __declspec(dllexport)
#else
# define SQLCLASS __declspec(dllimport)
# pragma comment(lib, "SqlAutmn")
#endif

class SQLCLASS SQL_Cursor;
class SQLCLASS SQL_Row;
class SQLCLASS SQL_Field;
struct SQL_TypeMethods;

class SQLCLASS SQL_Connection
{
public:
	// platform types
	enum PlatformType
	{
		PT_Default,
		PT_Oracle8
	};

	// construction/deconstruction
	SQL_Connection( const char *URL_ = NULL );
	SQL_Connection( const char *DB_, const char *USR_, const char *PSW_ );
	virtual ~SQL_Connection();
	virtual PlatformType GetPlatformType() const = 0;

	typedef void (*FN_ErrorHandler)( int CODE_, const char *TEXT_,
							SQL_Connection *, SQL_Cursor * );
	// error handling
	virtual void SetErrorHandler( FN_ErrorHandler HANDLER_ ) = 0;
	virtual void HandleError( int CODE_, const char *TEXT_, SQL_Cursor *CURSOR_ );
	virtual int GetErrorText( char *TEXT_, int MAX_SIZE_ ) = 0;

	// creating objects
	virtual SQL_Connection *CreateConnection( bool INHERIT_PROPERTIES_ ) const = 0;
	virtual SQL_Cursor *CreateCursor() = 0;

	// connection/disconnection
	virtual void SetURL( const char *URL_ );
	virtual const char *GetURL() const;
	virtual bool Connect( const char *URL_ = NULL ) = 0;
	virtual void Disconnect() = 0;

protected:
	char *_URL;
};

union set_id
{
	int			number;
	const char	*name;

	set_id( unsigned int number_ )
		: number(number_ >= 65536 ? 0 : number_) {}
	set_id( const char *name_ )
		: name(name_) {}
};

struct SQL_Long
{
	const char	*column_name;
	void		*data;

	SQL_Long( const char *name_ = NULL ) : column_name(name_), data(NULL) {}
	void SetColumn( const char *name_ ) { column_name = name_; }
};

class SQLCLASS SQL_Cursor
{
public:
// construction/deconstruction
	SQL_Cursor( SQL_Connection *CONNECTION_ = NULL );
	virtual ~SQL_Cursor();

// error handling
	virtual void HandleError( int CODE_, const char *TEXT_ );

// connections
	virtual SQL_Connection *GetConnection() const;
	virtual bool Connect( SQL_Connection *CONNECTION_ = NULL ) = 0;
	virtual void Disconnect() = 0;
	virtual bool IsConnected() const = 0;

// transaction control
	virtual bool Commit() = 0;
	virtual bool Rollback() = 0;
// execution
	virtual bool Reset() = 0;
	virtual bool CompileSQL( const char *QUERY_ ) = 0;
	virtual bool ExecuteSQL( const char *QUERY_ = NULL ) = 0;
	virtual bool ExecuteProcedure( const char *COMMAND_ ) = 0;

// fetching data
	virtual bool FetchNext( SQL_Row *ROW_ ) = 0;
	virtual bool CancelQuery() = 0;

// long column operations
	virtual unsigned int LongWrite( const SQL_Long &LONG_,
					const void *DATA_, unsigned int SIZE_ ) = 0;
	virtual unsigned int LongRead( const SQL_Long &LONG_,
					void *DATA_, unsigned int SIZE_ ) = 0;
	virtual bool LongEnd( const SQL_Long &LONG_ ) = 0;
	virtual bool LongComplete() = 0;

// operators
	virtual SQL_Cursor &pascal operator <<( set_id ) = 0;
	virtual SQL_Cursor &pascal operator <<( int ) = 0;
	virtual SQL_Cursor &pascal operator <<( unsigned int ) = 0;
	virtual SQL_Cursor &pascal operator <<( double ) = 0;
	virtual SQL_Cursor &pascal operator <<( const char * ) = 0;
	virtual SQL_Cursor &pascal operator <<( const SQL_Row & ) = 0;
	virtual SQL_Cursor &pascal operator <<( SQL_Long & ) = 0;

// inlines
protected:
// fields
	SQL_Connection *_Connection;
};

class SQLCLASS SQL_Field
{
public:
	enum Type
		{ T_Void, T_Number, T_Integer, T_Double, T_String, T_Date, T_Blob };
// construction/destruction
	SQL_Field() : _Column(0), _Data(NULL), _Methods(NULL)
		{}
	virtual ~SQL_Field()
		{}
// methods
	bool IsNull() const
		{ return !_Methods || _Methods->IsNull(_Data, _Column); }
	Type GetType() const
		{ return _Methods ? (Type)_Methods->GetType(_Data, _Column) : T_Void; }
	bool IsNumber() const
		{ return _Methods ? _Methods->IsNumber(_Data, _Column) : false; }
// operators
	operator int() const
		{ return _Methods ? _Methods->GetInt(_Data, _Column) : 0; }
	operator double() const
		{ return _Methods ? _Methods->GetDouble(_Data, _Column) : 0.0; }
	operator const char *() const
		{ return _Methods ? _Methods->GetString(_Data, _Column) : (char*)&_Methods; }
	operator SQL_Long() const
		{ return _Methods ? _Methods->GetLong(_Data, _Column) : *(SQL_Long*)NULL; }
	// date
protected:
	friend class SQL_Row;
	friend class SQL_Cursor_Oracle8;
	friend class SQL_Cursor_SQLBase;
	// internal structure used by SQL_Cursor_* only
	struct SQL_TypeMethods
	{
		// buffer operations
		void		(*UseData)( void *DATA_ );
		void		(*DisposeData)( void **DATA_ );
		int			(*GetCount)( void *DATA_ );
		// result item operations
		bool		(*IsNull)( void *DATA_, int COLUMN_ );
		Type		(*GetType)( void *DATA_, int COLUMN_ );
		bool		(*IsNumber)( void *DATA_, int COLUMN_ );
		int			(*GetInt)( void *DATA_, int COLUMN_ );
		double		(*GetDouble)( void *DATA_, int COLUMN_ );
		const char	*(*GetString)( void *DATA_, int COLUMN_ );
		SQL_Long	(*GetLong)( void *DATA_, int COLUMN_ );
	};
// control
	virtual void Initialize( int column_, void *data_, SQL_TypeMethods *methods_ )
		{
			_Column = column_;
			_Data = data_;
			_Methods = methods_;
		}
protected:
// fields
	int				_Column;
	void			*_Data;
	SQL_TypeMethods	*_Methods;
};

class SQLCLASS SQL_Row
{
public:
// construction/deconstruction
	SQL_Row() : _Cursor(NULL), _Fields(NULL), _Data(NULL), _Methods(NULL)
		{}
	virtual ~SQL_Row()
		{
			if (_Fields)
				delete[] _Fields;
			if (_Data && _Methods)
				_Methods->DisposeData(&_Data);
		}
// methods
	int GetCount() const
		{ return _Methods ? _Methods->GetCount(_Data) : 0; }
// operators
	const SQL_Field &operator []( int column_ ) const
		{ return _Fields[column_]; }
protected:
	friend class SQL_Cursor_Oracle8;
	friend class SQL_Cursor_SQLBase;
// control
	virtual void *GetData() const
		{ return _Data; }
	virtual void Initialize( SQL_Cursor *cursor_, int count_, void *data_,
		SQL_Field::SQL_TypeMethods *methods_, SQL_Field **fields_ )
		{
			if (data_ == _Data)
				return;
			if (_Fields)
				delete[] _Fields;
			methods_->DisposeData(&_Data);
			_Cursor = cursor_;
			_Data = data_;
			_Methods = methods_;
			*fields_ = _Fields = (count_ <= 0 ? NULL : new SQL_Field[count_+1]);
			methods_->UseData(data_);
		}
protected:
	SQL_Cursor					*_Cursor;
	SQL_Field					*_Fields;
	void						*_Data;
	SQL_Field::SQL_TypeMethods	*_Methods;
};

extern "C"
{

SQLCLASS SQL_Connection	*
	SQL_CreateConnection( SQL_Connection::PlatformType PLATFORM_, const char *URL_ );

SQLCLASS void
	SQL_DestroyConnection( SQL_Connection *CONNECTION_ );

SQLCLASS SQL_Cursor *
	SQL_CreateCursor( SQL_Connection *CONNECTION_ );

SQLCLASS void
	SQL_DestroyCursor( SQL_Cursor *CURSOR_ );

};

#pragma pack(pop)

#endif
